@@ -12,6 +12,7 @@ import contextlib
 import datetime
 import json
 import mimetypes
+import os
 import os.path
 import sys
 from dataclasses import dataclass
@@ -41,14 +42,6 @@ def base64_input(s: str) -> Base64:
     return Base64.from_encoded(s, ignore_whitespace=True)
 
 
-API_CERTIFICATE_MIME_TYPE = "application/x-pem-file"
-KEY_CERTIFICATE_MIME_TYPES = [
-    "application/x-pem-file",
-    "application/x-x509-ca-cert",
-    "application/pgp-keys",
-]
-
-
 DATETIME_TYPE = click.DateTime(formats=["%Y-%m-%dT%H:%M:%S%z"])
 ROLE_TYPE = make_enum_type(nethsm_sdk.Role)
 LOG_LEVEL_TYPE = make_enum_type(nethsm_sdk.LogLevel)
@@ -93,16 +86,14 @@ def print_table(headers: Sequence[str], data: Iterable[Sequence[Any]]) -> None:
 
 @dataclass
 class Config:
-    host: str
+    host: Optional[str]
     username: Optional[str]
     password: Optional[str]
     verify_tls: bool
 
 
 @click.group()
-@click.option(
-    "-h", "--host", "host", required=True, help="Set the host of the NetHSM API"
-)
+@click.option("-h", "--host", "host", help="Set the host of the NetHSM API")
 @click.option("-u", "--username", "username", help="The NetHSM user name")
 @click.option("-p", "--password", "password", help="The NetHSM password")
 @click.option(
@@ -113,12 +104,13 @@ class Config:
 @click.pass_context
 def nethsm(
     ctx: Context,
-    host: str,
+    host: Optional[str],
     username: Optional[str],
     password: Optional[str],
     verify_tls: bool,
 ) -> None:
     """Interact with NetHSM devices, see subcommands."""
+
     ctx.obj = Config(
         host=host, username=username, password=password, verify_tls=verify_tls
     )
@@ -129,22 +121,31 @@ def connect(ctx: Context, require_auth: bool = True) -> Iterator[NetHSM]:
     config = ctx.obj
     assert isinstance(config, Config)
 
+    host = config.host
+    if host is None:
+        v = "NETHSM_HOST"
+        if v not in os.environ:
+            raise CliException(
+                "Missing NetHSM host: set the --host option or the "
+                f"{v} environment variable",
+                support_hint=False,
+            )
+        host = os.environ[v]
+
     auth = None
     if require_auth:
         username = config.username
         password = config.password
         if not username:
-            username = prompt_str(f"[auth] User name for NetHSM {config.host}")
+            username = prompt_str(f"[auth] User name for NetHSM {host}")
         if not password:
             password = prompt_str(
-                f"[auth] Password for user {username} on NetHSM {config.host}",
+                f"[auth] Password for user {username} on NetHSM {host}",
                 hide_input=True,
             )
         auth = Authentication(username=username, password=password)
 
-    with nethsm_sdk.connect(
-        config.host, auth=auth, verify_tls=config.verify_tls
-    ) as nethsm:
+    with nethsm_sdk.connect(host, auth=auth, verify_tls=config.verify_tls) as nethsm:
         try:
             yield nethsm
         except nethsm_sdk.NetHSMError as e:
@@ -679,9 +680,14 @@ def prompt_mechanisms(type: str) -> list[str]:
             show_choices=False,
             show_default=False,
         )
+
+        if "" not in available_mechanisms:
+            available_mechanisms.append("")
+
         assert isinstance(mechanism, str)
         if mechanism:
             mechanisms.append(mechanism)
+            available_mechanisms.remove(mechanism)
         else:
             cont = False
 
@@ -906,7 +912,15 @@ def generate_key(
 @click.option("--public-key", is_flag=True, help="Query the public key")
 @click.option("--certificate", is_flag=True, help="Query the certificate")
 @click.pass_context
-def get_config(ctx: Context, **kwargs: bool) -> None:
+def get_config(
+    ctx: Context,
+    logging: bool,
+    network: bool,
+    time: bool,
+    unattended_boot: bool,
+    public_key: bool,
+    certificate: bool,
+) -> None:
     """Query the configuration of a NetHSM.
 
     Only the configuration items selected with the corresponding option are
@@ -916,40 +930,42 @@ def get_config(ctx: Context, **kwargs: bool) -> None:
     role."""
     with connect(ctx) as nethsm:
         print(f"Configuration for NetHSM {nethsm.host}:")
-        show_all = not any(kwargs.values())
+        show_all = not any(
+            [logging, network, time, unattended_boot, public_key, certificate]
+        )
 
-        if show_all or kwargs["logging"]:
-            logging = nethsm.get_config_logging()
+        if show_all or logging:
+            logging_config = nethsm.get_config_logging()
             print("  Logging:")
-            print("    IP address:   ", logging.ip_address)
-            print("    Port:         ", logging.port)
-            print("    Log level:    ", logging.log_level)
+            print("    IP address:   ", logging_config.ip_address)
+            print("    Port:         ", logging_config.port)
+            print("    Log level:    ", logging_config.log_level)
 
-        if show_all or kwargs["network"]:
-            network = nethsm.get_config_network()
+        if show_all or network:
+            network_config = nethsm.get_config_network()
             print("  Network:")
-            print("    IP address:   ", network.ip_address)
-            print("    Netmask:      ", network.netmask)
-            print("    Gateway:      ", network.gateway)
+            print("    IP address:   ", network_config.ip_address)
+            print("    Netmask:      ", network_config.netmask)
+            print("    Gateway:      ", network_config.gateway)
 
-        if show_all or kwargs["time"]:
-            time = nethsm.get_config_time()
-            print("  Time:           ", time)
+        if show_all or time:
+            time_config = nethsm.get_config_time()
+            print("  Time:           ", time_config)
 
-        if show_all or kwargs["unattended_boot"]:
-            unattended_boot = nethsm.get_config_unattended_boot()
-            print("  Unattended boot:", unattended_boot)
+        if show_all or unattended_boot:
+            unattended_boot_config = nethsm.get_config_unattended_boot()
+            print("  Unattended boot:", unattended_boot_config)
 
-        if show_all or kwargs["public_key"]:
-            public_key = nethsm.get_public_key()
+        if show_all or public_key:
+            public_key_config = nethsm.get_public_key()
             print("  Public key:")
-            for line in public_key.splitlines():
+            for line in public_key_config.splitlines():
                 print(f"    {line}")
 
-        if show_all or kwargs["certificate"]:
-            certificate = nethsm.get_certificate()
+        if show_all or certificate:
+            certificate_config = nethsm.get_certificate()
             print("  Certificate:")
-            for line in certificate.splitlines():
+            for line in certificate_config.splitlines():
                 print(f"    {line}")
 
 
