@@ -78,6 +78,12 @@ def _list() -> None:
     is_flag=True,
     help="Allow updates with an outdated pynitrokey version (dangerous)",
 )
+@click.option(
+    "--confirm",
+    default=False,
+    is_flag=True,
+    help="Confirm all questions to allow running non-interactively",
+)
 @click.option(
     "--experimental",
     default=False,
@@ -91,15 +97,17 @@ def update(
     image: Optional[str],
     version: Optional[str],
     ignore_pynitrokey_version: bool,
+    confirm: bool,
     experimental: bool,
 ) -> None:
     """
     Update the firmware of the device using the given image.
 
     This command requires that exactly one Nitrokey 3 in bootloader or firmware mode is connected.
-    The user is asked to confirm the operation before the update is started.  The Nitrokey 3 may
-    not be removed during the update.  Also, additional Nitrokey 3 devices may not be connected
-    during the update.
+    The user is asked to confirm the operation before the update is started.  If the --confirm
+    option is provided, this is the confirmation.  This option may be used to automate an update.
+    The Nitrokey 3 may not be removed during the update.  Also, additional Nitrokey 3 devices may
+    not be connected during the update.
 
     If no firmware image is given, the latest firmware release is downloaded automatically.  If
     the --version option is set, the given version is downloaded instead.
@@ -115,7 +123,7 @@ def update(
 
     from .update import update as exec_update
 
-    exec_update(ctx, image, version, ignore_pynitrokey_version)
+    exec_update(ctx, image, version, ignore_pynitrokey_version, confirm)
 
 
 @nk3.command()
@@ -164,23 +172,39 @@ def set_config(ctx: Context, key: str, value: str, force: bool, dry_run: bool) -
     """
 
     with ctx.connect_device() as device:
-        # before the confirmation prompt, check if the config value is supported
-        if not device.admin.has_config(key):
+        config_fields = device.admin.list_available_fields()
+
+        field_metadata = None
+        for field in config_fields:
+            if field.name == key:
+                field_metadata = field
+
+        if field_metadata is None:
+            print(
+                "Changing configuration values can have unexpected side effects, including data loss.",
+                file=sys.stderr,
+            )
+            print(
+                "This should only be used for development and testing.",
+                file=sys.stderr,
+            )
+            if not force:
+                raise CliException(
+                    "Unknown config values can only be set if the --force/-f flag is set.  Aborting.",
+                    support_hint=False,
+                )
+
+        if (
+            not force
+            and field_metadata is not None
+            and not field_metadata.ty.is_valid(value)
+        ):
             raise CliException(
-                f"The configuration option '{key}' is not supported by the device.",
+                f"Invalid config value for {field}: expected {field_metadata.ty}, got `{value}`. Unknown config values can only be set if the --force/-f flag is set.  Aborting.",
                 support_hint=False,
             )
 
-        # config fields that don’t have side effects
-        whitelist = [
-            "fido.disable_skip_up_timeout",
-        ]
-        requires_touch = False
-        requires_reboot = False
-
         if key == "opcard.use_se050_backend":
-            requires_touch = True
-            requires_reboot = True
             print(
                 "This configuration values determines whether the OpenPGP Card "
                 "application uses a software implementation or the secure element.",
@@ -192,31 +216,20 @@ def set_config(ctx: Context, key: str, value: str, force: bool, dry_run: bool) -
                 "user data currently stored on the device.",
                 file=sys.stderr,
             )
-        elif key not in whitelist:
-            pass
+        elif field_metadata is not None and field_metadata.destructive:
             print(
-                "Changing configuration values can have unexpected side effects, including data loss.",
-                file=sys.stderr,
-            )
-            print(
-                "This should only be used for development and testing.",
+                "This configuration value may delete data on your device",
                 file=sys.stderr,
             )
 
-            if not force:
-                raise CliException(
-                    "Unknown config values can only be set if the --force/-f flag is set.  Aborting.",
-                    support_hint=False,
-                )
-
-        if key not in whitelist:
+        if field_metadata is not None and field_metadata.destructive:
             click.confirm("Do you want to continue anyway?", abort=True)
 
         if dry_run:
             print("Stopping dry run.", file=sys.stderr)
             raise click.Abort()
 
-        if requires_touch:
+        if field_metadata is not None and field_metadata.requires_touch_confirmation:
             print(
                 "Press the touch button to confirm the configuration change.",
                 file=sys.stderr,
@@ -224,7 +237,7 @@ def set_config(ctx: Context, key: str, value: str, force: bool, dry_run: bool) -
 
         device.admin.set_config(key, value)
 
-        if requires_reboot:
+        if field_metadata is not None and field_metadata.requires_reboot:
             print("Rebooting device to apply config change.")
             device.reboot()
 
